@@ -80,10 +80,10 @@ static OptsParser::Option options[] = {
       "Generate manipulated header files", OptsParser::AT_NONE },
    { AGxxConfig::ACOPT_ASPECT_HEADER, "a", "aspect_header",
       "Name of aspect header file or '0'", OptsParser::AT_MANDATORY },
-   { AGxxConfig::ACOPT, "r", "repository",
+   { AGxxConfig::ACOPT_QUOTE_ARG, "r", "repository",
       "Name of the project repository", OptsParser::AT_MANDATORY },
    { AGxxConfig::ACOPT_MATCH_POINTCUT_EXPR, NULL, "expr",
-      "Pointcut expression to match in repository", OptsParser::AT_MANDATORY},
+      "\t" "Pointcut expression to match in repository", OptsParser::AT_MANDATORY},
    { AGxxConfig::ACOPT, NULL, "problem_local_class",
       "Back-end does not support local classes correctly", OptsParser::AT_NONE },
    { AGxxConfig::ACOPT, NULL, "no_problem_local_class",
@@ -104,9 +104,13 @@ static OptsParser::Option options[] = {
       "\t" "Warn if a macro transformation had to be ignored", OptsParser::AT_NONE },
    { AGxxConfig::ACOPT, NULL, "no_warn_macro",
       "\t" "Don't warn if a macro transformation had to be ignored", OptsParser::AT_NONE },
+   { AGxxConfig::ACOPT, NULL, "warn_limitations",
+     "" "Warn if a feature does not work as familiar due to limitations", OptsParser::AT_NONE },
+   { AGxxConfig::ACOPT, NULL, "no_warn_limitations",
+     "" "Don't warn if a feature does not work as familiar due to limitations", OptsParser::AT_NONE },
    { AGxxConfig::ACOPT, NULL,"no_line",
       "\t\t" "Don't generate #line directives",OptsParser::AT_NONE },
-   { AGxxConfig::ACOPT, NULL, "gen_size_type",
+   { AGxxConfig::ACOPT_QUOTE_ARG, NULL, "gen_size_type",
       "Argument type for generated new operator", OptsParser::AT_MANDATORY},
    { AGxxConfig::ACOPT, NULL,"real-instances",
       "\t" "Perform full template analysis",OptsParser::AT_NONE },
@@ -118,6 +122,10 @@ static OptsParser::Option options[] = {
       "\t" "Path to destination for modified include files", OptsParser::AT_MANDATORY},
    { AGxxConfig::ACOPT_CFG_FILE, NULL, "config",
       "\t" "Full path to a config file", OptsParser::AT_MANDATORY},
+   { AGxxConfig::ACOPT_SYS_CFG_FILE, NULL, "system-config",
+      "Full path to a system config file (lower priority options)", OptsParser::AT_MANDATORY},
+   { AGxxConfig::AGOPT_TARGET, NULL, "target",
+      "\t" "Override the compiler's target triple", OptsParser::AT_MANDATORY},
    { AGxxConfig::ACOPT_PRE_INCLUDE, "I", NULL,
       "\t\t" "Add new include path", OptsParser::AT_MANDATORY},
    { AGxxConfig::ACOPT_PRE_DEFINE, "D", NULL,
@@ -132,8 +140,16 @@ static OptsParser::Option options[] = {
       "\t\t" "Prepare generated code for dynamic weaving", OptsParser::AT_NONE },
    { AGxxConfig::ACOPT, NULL, "introspection",
       "\t" "Support introspection mechanism", OptsParser::AT_NONE },
+   { AGxxConfig::ACOPT, NULL, "flow_analysis",
+      "\t" "Support control-flow analysis and data-flow analysis", OptsParser::AT_NONE },
+   { AGxxConfig::ACOPT, NULL, "builtin_operators",
+      "Support joinpoints on builtin operator calls", OptsParser::AT_NONE },
+   { AGxxConfig::ACOPT, NULL, "data_joinpoints",
+      "\t" "Support data-based joinpoints like get(), set(), ...", OptsParser::AT_NONE },
    { AGxxConfig::ACOPT_SYS_INCLUDE, "isystem", NULL,
-      "\t\t" "Add system include path", OptsParser::AT_MANDATORY},
+      "\t" "Add system include path", OptsParser::AT_MANDATORY},
+   { AGxxConfig::ACOPT_QUOTE_ARG, NULL, "warn_compatibility",
+      "Enable migration warnings for code based on major version <arg>", OptsParser::AT_MANDATORY },
    { 0, 0, 0, 0, OptsParser::AT_NONE}
 };
 
@@ -304,7 +320,16 @@ AGxxConfig::parseOptions()
         _x_compiler = false;
         continue;
 
+      case AGOPT_TARGET:
+        _command_line_target_triple = op.getArgument();
+        continue;
+
       case ACOPT_CFG_FILE:
+        _optvec.pushback("--config", " \"" + op.getArgument() + "\" ",
+            (OptionItem::OPT_ACC));
+        continue;
+
+      case ACOPT_SYS_CFG_FILE:
         _pumaconfig_file = op.getArgument();
         _pumaconfig_tmp = false;
         continue;
@@ -430,11 +455,19 @@ AGxxConfig::parseOptions()
         _config_command = op.getArgument();
         break;
 
-      case ACOPT: // default handling for Aspect C++ options
+      // default handling for Aspect C++ options
+      case ACOPT:
         _optvec.pushback(op.getOptionName(), op.getArgument(),
             OptionItem::OPT_ACC);
         break;
 
+      // default handling for Aspect C++ options
+      // where the argument has to be quoted
+      case ACOPT_QUOTE_ARG:
+        _optvec.pushback(op.getOptionName(), " \""+op.getArgument()+"\"",
+            OptionItem::OPT_ACC);
+        break;
+
       case ACOPT_WITHOUT_INPUT:
         _run_acpp_without_input = true;
         _weave_only = true;
