@@ -24,28 +24,37 @@ using std::stringstream;
 using std::set;
 
 // PUMA includes
-#include "Puma/ExtACKeywords.ah"
-#include "Puma/ErrorSeverity.h"
-#include "Puma/OptsParser.h"
 #include "Puma/SysCall.h"
-#include "Puma/StrCol.h"
+
+// Clang includes
+#ifdef FRONTEND_CLANG
+#include "clang/Basic/TargetInfo.h"
+#include "clang/Frontend/CompilerInstance.h"
+#include "clang/Basic/Version.h"
+#else
+#include "Puma/CTypeInfo.h"
+#endif // FRONTEND_CLANG
 
 // AspectC++ includes
 #include "ACConfig.h"
 #include "version.h"
 #include "Naming.h"
 
+using namespace Puma;
+
 static OptsParser::Option options[] = {
   { ACConfig::ACOPT_VERSION, "V", "version",
     "\t" "Version information", OptsParser::AT_NONE },
+  { ACConfig::ACOPT_HELP, "h", "help",
+    "\t" "\t" "Get this help text", OptsParser::AT_NONE },
   { ACConfig::ACOPT_VERBOSE, "v", "verbose",
-    "Level of verbosity (0-9)", OptsParser::AT_MANDATORY },
+    "\t" "Level of verbosity (0-9)", OptsParser::AT_MANDATORY },
   { ACConfig::ACOPT_COMPILE, "c", "compile",
-    "Name of the input file", OptsParser::AT_MANDATORY },
+    "\t" "Name of the input file", OptsParser::AT_MANDATORY },
   { ACConfig::ACOPT_OUTPUT, "o", "output",
     "\t" "Name of the output file", OptsParser::AT_MANDATORY },
   { ACConfig::ACOPT_INCLUDE_FILES, "i", "include_files",
-    "Generate manipulated header files", OptsParser::AT_NONE },
+    "\t" "Generate manipulated header files", OptsParser::AT_NONE },
   { ACConfig::ACOPT_ASPECT_HEADER, "a", "aspect_header",
     "Name of aspect header file or '0'", OptsParser::AT_MANDATORY }, 
   { ACConfig::ACOPT_REPOSITORY, "r", "repository",
@@ -86,6 +95,12 @@ static OptsParser::Option options[] = {
   { ACConfig::ACOPT_NO_WARN_MACRO, NULL, "no_warn_macro",
     "\t" "Don't warn if a macro transformation had to be ignored",
     OptsParser::AT_NONE },
+  { ACConfig::ACOPT_WARN_LIMITATIONS, NULL, "warn_limitations",
+    "" "Warn if a feature does not work as familiar due to limitations",
+    OptsParser::AT_NONE },
+  { ACConfig::ACOPT_NO_WARN_LIMITATIONS, NULL, "no_warn_limitations",
+    "" "Don't warn if a feature does not work as familiar due to limitations",
+    OptsParser::AT_NONE },
   { ACConfig::ACOPT_KEYWORDS, "k", "keywords",
     "\t" "Allow AspectC++ keywords in normal project files",
     OptsParser::AT_NONE },
@@ -93,10 +108,22 @@ static OptsParser::Option options[] = {
     "\t\t" "Prepare generated code for dynamic weaving",
     OptsParser::AT_NONE },
   { ACConfig::ACOPT_INTROSPECTION, 0, "introspection",
-    "\t\t" "Support introspection mechanism",
+    "\t" "Support introspection mechanism",
+    OptsParser::AT_NONE },
+  { ACConfig::ACOPT_FLOW_ANALYSIS, 0, "flow_analysis",
+    "\t" "Support control-flow analysis and data-flow analysis",
+    OptsParser::AT_NONE },
+  { ACConfig::ACOPT_BUILTIN_OPERATORS, 0, "builtin_operators",
+    "Support joinpoints on builtin operator calls",
+    OptsParser::AT_NONE },
+  { ACConfig::ACOPT_DATA_JOINPOINTS, 0, "data_joinpoints",
+    "\t" "Support data-based joinpoints like get(), set(), ...",
     OptsParser::AT_NONE },
   { ACConfig::ACOPT_INTRODUCTION_DEPTH, 0, "introduction_depth",
     "Set the maximum depth for nested introductions", OptsParser::AT_MANDATORY },
+  { ACConfig::ACOPT_WARN_COMPATIBILITY, NULL, "warn_compatibility",
+    "Enable migration warnings for code based on major version <arg>",
+    OptsParser::AT_MANDATORY },
   { 0, NULL, 0, 0, OptsParser::AT_NONE }
 };
 
@@ -111,6 +138,24 @@ ACConfig::~ACConfig () {
 }
 
 
+string ACConfig::size_type () const {
+  string result = _size_type;
+  // if the size-type string wasn't provided with the command line option
+  // --gen_size_type, ask the front end for its target size type.
+  if (result == "") {
+#ifdef FRONTEND_CLANG
+    clang::CompilerInstance *ci = _project.get_compiler_instance ();
+    clang::TargetInfo &target = ci->getTarget ();
+    result = clang::TargetInfo::getTypeName (target.getSizeType ());
+#else
+    ostringstream size_type_stream;
+    size_type_stream << *CTypeInfo::CTYPE_SIZE_T;
+    result = size_type_stream.str ();
+#endif
+  }
+  return result;
+}
+
 bool ACConfig::analyze () {
 
   // setup defaults
@@ -135,13 +180,18 @@ bool ACConfig::analyze () {
   } else if (_project.config ().Option ("--vc")) {
     _problem_spec_scope          = false;
   }
-  _warn_deprecated    = true;
-  _warn_macro         = false;
-  _dynamic            = false;
-  _introspection      = false;
-  bool keywords      = false;
-  _introduction_depth = 10;
-  
+  _warn_deprecated     = true;
+  _warn_macro          = false;
+  _warn_limitations    = true;
+  _dynamic             = false;
+  _introspection       = false;
+  _flow_analysis       = false;
+  _builtin_operators   = false;
+  _data_joinpoints     = false;
+  _keywords            = false;
+  _introduction_depth  = 10;
+  _warn_compatibility = -1;
+
   char* tmp;
 
   OptsParser op (_argc, _argv, options);
@@ -151,28 +201,35 @@ bool ACConfig::analyze () {
     switch (op.getResult ()) {
     case ACOPT_VERBOSE:
       if (op.getArgument ().empty ())
-	// -v without number
-      	_v = 3;
+        // -v without number
+        _v = 3;
       else
-      	_v = atoi (op.getArgument ().c_str ());
+        _v = atoi (op.getArgument ().c_str ());
       break;
     case ACOPT_VERSION:
       cout << ac_program () << " " << ac_version ()
-           << " (" << __DATE__ << ")" << endl;
-      return false;
+           << " (" << __DATE__;
+#ifdef FRONTEND_CLANG
+      cout << ", clang " CLANG_VERSION_STRING;
+#endif // FRONTEND_CLANG
+      cout << ")" << endl;
+      return true;
+    case ACOPT_HELP:
+      usage (_argv[0]);
+      return true;
     case ACOPT_COMPILE:
-  	  tmp = StrCol::dup (op.getArgument ().c_str ());
+      tmp = StrCol::dup (op.getArgument ().c_str ());
 #if defined (WIN32)
-  	  SysCall::MakeUnixPath(tmp);
+      SysCall::MakeUnixPath(tmp);
 #endif
-  		_file_in = tmp;
+      _file_in = tmp;
       break;
     case ACOPT_OUTPUT:
-  	  tmp = StrCol::dup (op.getArgument ().c_str ());
+      tmp = StrCol::dup (op.getArgument ().c_str ());
 #if defined (WIN32)
-  	  SysCall::MakeUnixPath(tmp);
+      SysCall::MakeUnixPath(tmp);
 #endif
-  		_file_out = tmp;
+      _file_out = tmp;
       break;
     case ACOPT_INCLUDE_FILES:
       _ifiles = true;
@@ -240,8 +297,17 @@ bool ACConfig::analyze () {
     case ACOPT_NO_WARN_MACRO:
       _warn_macro = false;
       break;
+    case ACOPT_WARN_LIMITATIONS:
+      _warn_limitations = true;
+      break;
+    case ACOPT_NO_WARN_LIMITATIONS:
+      _warn_limitations = false;
+      break;
+    case ACOPT_WARN_COMPATIBILITY:
+      _warn_compatibility = atoi (op.getArgument ().c_str ());
+      break;
     case ACOPT_KEYWORDS:
-      keywords = true;
+      _keywords = true;
       break;
     case ACOPT_DYNAMIC:
       _dynamic = true;
@@ -249,11 +315,29 @@ bool ACConfig::analyze () {
     case ACOPT_INTROSPECTION:
       _introspection = true;
       break;
+    case ACOPT_FLOW_ANALYSIS:
+      _flow_analysis = true;
+      break;
+#ifdef FRONTEND_PUMA
+    case ACOPT_BUILTIN_OPERATORS:
+    case ACOPT_DATA_JOINPOINTS:
+      err() << sev_error << "The command line options '--builtin_operators' "
+          "and '--data_joinpoints' are not supported in the 'Puma' configuration." << endMessage;
+      return false;
+#else
+    case ACOPT_BUILTIN_OPERATORS:
+      _builtin_operators = true;
+      break;
+    case ACOPT_DATA_JOINPOINTS:
+      _data_joinpoints = true;
+      break;
+#endif
     case ACOPT_INTRODUCTION_DEPTH:
       _introduction_depth = atoi (op.getArgument ().c_str ());
       break;
     default:
       usage (_argv[0]);
+      err () << sev_error;
       return false;
     }
   }
@@ -266,6 +350,7 @@ bool ACConfig::analyze () {
   if (!expr().empty ()) {
     if (!repository()) {
       usage (_argv[0]);
+      err () << sev_error;
       return false;
     }
     return true;
@@ -277,16 +362,25 @@ bool ACConfig::analyze () {
       (_file_out && have_dest) ||
       (!_file_in && !_ifiles && !have_dest)) {
     usage (_argv[0]);
+    err () << sev_error;
     return false;
   }
 
+  // check option dependency
+  if( _data_joinpoints ) {
+    if( ! _builtin_operators ) {
+      err() << sev_warning << "--data_joinpoints internally requires --builtin_operators, please check and enable it to silence this warning!" << endMessage;
+      _builtin_operators = true;
+    }
+  }
+
   // check for the compatibility mode (iterate through the project tree)
   if (!_ifiles && !_file_in && !_file_out && have_dest)
     _iterate_tunits = true;
 
   // set reasonable default values
   if (!_file_out && _file_in)
-		_file_out = StrCol::dup ("ac.out");
+    _file_out = StrCol::dup ("ac.out");
 
   if (!_file_in) {
     if (_file_out)
@@ -296,7 +390,7 @@ bool ACConfig::analyze () {
       if (_project.src (0L))
         genfile << _project.src (0L);
       genfile << "ac_gen." << extension ();
-	   _file_in = StrCol::dup (genfile.str ().data ());
+      _file_in = StrCol::dup (genfile.str ().data ());
     }
   }
 
@@ -327,9 +421,6 @@ bool ACConfig::analyze () {
     _project_id = name.str ();
   }
   
-  // select AspectC++ as the input language
-  ExtACKeywords::aspectof ()->prj_keywords (keywords);
-  
   // TODO: this should be obsolete -> check it
   _project.config ().Add ("--lang-ac++");
   _project.configure (_project.config ());
@@ -338,27 +429,26 @@ bool ACConfig::analyze () {
   return true;
 }
 
-void ACConfig::usage (const char *prog) const
- {
-   cerr << "usage: " << prog << " options" << endl
-        << "   options are ..." << endl;
-   cerr << "     -p|--path <arg>\t\t" << "project directory tree" << endl
-	<< "     -d|--dest <arg>\t\t" << "target directory tree" << endl
-	<< "     -e|--extension <arg>\t" << "translation unit filename ext."
-	<< endl;
-   for (int o = 0; options[o].number > 0; o++) {
-     cerr << "     ";
-     if (options[o].shortName)  cerr << "-" << options[o].shortName;
-     if (options[o].shortName && options[o].longName) cerr << "|";
-     if (options[o].longName) cerr << "--" << options[o].longName;
-     if (options[o].argType != OptsParser::AT_NONE) cerr << " <arg>";
-     if (options[o].desc) cerr << " \t" << options[o].desc;
-     cerr << endl;
-   }
-   cerr << "   further options for correct parsing are ..." << endl
-	<< "     -I <arg>\t\t\t" << "include file search path" << endl
-	<< "     -D <name>[=<value>]\t" << "macro definitions" << endl;
- }
+void ACConfig::usage (const char *prog) const {
+  cerr << "usage: " << prog << " options" << endl
+       << "   options are ..." << endl;
+  cerr << "     -p|--path <arg>\t\t" << "project directory tree" << endl
+       << "     -d|--dest <arg>\t\t" << "target directory tree" << endl
+       << "     -e|--extension <arg>\t" << "translation unit filename ext."
+       << endl;
+  for (int o = 0; options[o].number > 0; o++) {
+    cerr << "     ";
+    if (options[o].shortName)  cerr << "-" << options[o].shortName;
+    if (options[o].shortName && options[o].longName) cerr << "|";
+    if (options[o].longName) cerr << "--" << options[o].longName;
+    if (options[o].argType != OptsParser::AT_NONE) cerr << " <arg>";
+    if (options[o].desc) cerr << "\t" << options[o].desc;
+    cerr << endl;
+  }
+  cerr << "   further options for correct parsing are ..." << endl
+       << "     -I <arg>\t\t\t" << "include file search path" << endl
+       << "     -D <name>[=<value>]\t" << "macro definitions" << endl;
+}
 
 
 // get the extension string for cc files
@@ -368,7 +458,7 @@ const char *ACConfig::extension () {
     const ConfOption *o = _project.config ().Option (i-1);
     if (! strcmp (o->Name (), "-e") || ! strcmp (o->Name (), "--extension")) {
       if (! o->Arguments ())
-	continue;
+        continue;
       return o->Argument (0);
     }
   }
