@@ -23,7 +23,15 @@
 // complain about errors => result false
 bool MatchSignature::parse (const string &str) {
   const char *strptr = str.c_str();
-  if (parse_match_expr (strptr)) {
+  if( str == "::" ) { // special case for root namespace
+    _state = NORMAL;
+    _expr_type = TYPE;
+    MatchName n;
+    n.name( "::" );
+    _type = MatchTypeRef::make_named( n );
+    return true;
+  }
+  else if (parse_match_expr (strptr)) {
     // perform argument conversions, warn if any conversion are applied
     bool f = false, a = false, q = false, v = false;
     _type.adjust_args (f, a, q, v);
@@ -36,13 +44,19 @@ bool MatchSignature::parse (const string &str) {
   
 // internal parser functions that analyse a match expr string
 bool MatchSignature::parse_match_expr (const char *&str) {
-  MatchTypeRef type;
-  if (!parse_declaration (str, false, type)) {
-    if (!error ())
-      _state = ERR_EMPTY_MATCH_EXPR;
+  // empty strings are no valid match expressions
+  if (*str == '\0') {
+    _state = ERR_EMPTY_MATCH_EXPR;
     return false;
   }
-  else if (*str != '\0') {
+
+  // parse as a C++ declaration
+  MatchTypeRef type;
+  if (!parse_declaration (str, false, type))
+    return false;
+
+  // check for tokens after the end of the accepted input
+  if (*str != '\0') {
     _state = ERR_TOKENS_AFTER_END;
     return false;
   }
@@ -60,7 +74,8 @@ bool MatchSignature::parse_declaration (const char *&str, bool abstract,
                                    MatchTypeRef &type) {
   // decl_spec_seq_opt declarator_opt
   FctSpec fct_spec;
-  parse_decl_spec_seq (str, type, fct_spec);
+  Spec spec;
+  parse_decl_spec_seq( str, type, fct_spec, spec );
   if (error ())
     return false;
     
@@ -74,20 +89,22 @@ bool MatchSignature::parse_declaration (const char *&str, bool abstract,
     
   // function specifiers are only valid in conjunctions with function types
   if (!type.is_function() && fct_spec) {
-    _state = ERR_INVALID_FCT_SPEC;
+    _state = ERR_INVALID_SPEC;
     return false;
   }
   
   // function specifier in type that appear in argument lists etc. are ignored.
   // Top-level function specifiers are saved for matching functions
-  if (!abstract)
+  if (!abstract) {
     _fct_spec = fct_spec;
+    _spec = spec;
+  }
     
   return true;
 }
 
 bool MatchSignature::parse_decl_spec_seq (const char *&str, MatchTypeRef &type,
-  FctSpec &fs) {
+  FctSpec &fs, Spec &s ) {
 
   DeclSpecs ds; // semantic knowledge about the decl spec seq
   
@@ -105,8 +122,14 @@ bool MatchSignature::parse_decl_spec_seq (const char *&str, MatchTypeRef &type,
     // a named type ...
 
     // check for the any-type wildcard first
-    if (ds._name.scopes () == 0 && ds._name.name () == string("%"))
-      type = MatchTypeRef::make_any ();
+    if (ds._name.scopes () == 0 && ds._name.name () == string("%")) {
+      if (_wildcards_allowed)
+        type = MatchTypeRef::make_any ();
+      else {
+        _state = ERR_WILDCARD;
+        return false;
+      }
+    }
     else
       type = MatchTypeRef::make_named (ds._name);
   }
@@ -146,6 +169,8 @@ bool MatchSignature::parse_decl_spec_seq (const char *&str, MatchTypeRef &type,
         else if (ds._size == 2)
           type = MatchTypeRef::make_unsigned_long_long ();
       }
+      else if (ds._type == PRIM_INT128 && ds._size == 0)
+        type = MatchTypeRef::make_unsigned_int128 ();
     }
     else {
       // neither defined as signed nor unsigned
@@ -163,6 +188,8 @@ bool MatchSignature::parse_decl_spec_seq (const char *&str, MatchTypeRef &type,
         else if (ds._size == 2)
           type = MatchTypeRef::make_long_long ();
       }
+      else if (ds._type == PRIM_INT128 && ds._size == 0)
+        type = MatchTypeRef::make_int128 ();
       else if (ds._type == PRIM_WCHAR_T && ds._size == 0)
         type = MatchTypeRef::make_wchar_t ();
       else if (ds._type == PRIM_FLOAT && ds._size == 0)
@@ -192,6 +219,7 @@ bool MatchSignature::parse_decl_spec_seq (const char *&str, MatchTypeRef &type,
   
   // also return the parsed function specifiers
   fs = ds._fct_spec;
+  s = ds._spec;
   
   return true;
 }
@@ -220,21 +248,24 @@ bool MatchSignature::parse_decl_spec (const char *&str, DeclSpecs &ds) {
   
   // check for function specifiers; currently only virtual
   FctSpec fct_spec = FCT_NONE;
+  Spec spec = DT_NONE;
   if (next_word ("virtual", str))
     fct_spec = FCT_VIRTUAL;
   else if (next_word ("static", str))
-    fct_spec = FCT_STATIC;
-  if (fct_spec) {
-    if (ds._fct_spec & fct_spec) {
-      _state = ERR_DUP_FCT_SPEC;
+    spec = DT_STATIC;
+
+  if( fct_spec || spec ) {
+    if( ( ds._fct_spec & fct_spec ) || ( ds._spec & spec ) ) {
+      _state = ERR_DUP_SPEC;
       return false;
     }
-    else if (((ds._fct_spec & FCT_STATIC) && fct_spec == FCT_VIRTUAL) ||
-        ((ds._fct_spec & FCT_VIRTUAL) && fct_spec == FCT_STATIC)) {
+    else if (((ds._spec & DT_STATIC) && fct_spec == FCT_VIRTUAL) ||
+        ((ds._fct_spec & FCT_VIRTUAL) && spec == DT_STATIC)) {
       _state = ERR_VIRT_STAT_FCT_SPEC;
       return false;
     }
     ds._fct_spec = (FctSpec)(ds._fct_spec | fct_spec); // remember this fct spec
+    ds._spec = (Spec)(ds._spec | spec); // remember this spec
     return true;
   }
   
@@ -247,6 +278,8 @@ bool MatchSignature::parse_decl_spec (const char *&str, DeclSpecs &ds) {
     ds._type = PRIM_CHAR;
   else if (next_word ("int", str))
     ds._type = PRIM_INT;
+  else if (next_word ("__int128", str))
+    ds._type = PRIM_INT128;
   else if (next_word ("void", str))
     ds._type = PRIM_VOID;
   else if (next_word ("bool", str))
@@ -406,10 +439,11 @@ bool MatchSignature::parse_opt_template_argument_list (const char *&str,
   str++;
   
   mtal = new MatchTemplateArgList;
-  if (!parse_template_argument_list (str, *mtal));
+  if (!parse_template_argument_list (str, *mtal)) {
     if (error ()) {
-    delete mtal;
-    return false;
+      delete mtal;
+      return false;
+    }
   }
   if (*str != '>') {
     _state = ERR_INVALID_ARG_LIST;
@@ -463,6 +497,10 @@ bool MatchSignature::parse_template_argument (const char *&str,
   
   // now check for 'any'
   if (str[0] == '%') {
+    if (!_wildcards_allowed) {
+      _state = ERR_WILDCARD;
+      return false;
+    }
     const char *curr = str;
     curr++;
     skip_blanks (curr);
@@ -527,6 +565,9 @@ bool MatchSignature::parse_dec_literal (const char *&str, long long &val) {
     str++;
   }
   val *= sign;
+  // Skip ULL suffixes.
+  while (*str == 'U' || *str == 'L')
+    ++str;
   return true;
 }
 
@@ -664,6 +705,10 @@ bool MatchSignature::parse_declarator_post (const char *&str, MatchTypeRef &type
     bool any_size = false;
     unsigned long array_size = 0;
     if (*str == '%') {
+      if (!_wildcards_allowed) {
+        _state = ERR_WILDCARD;
+        return false;
+      }
       str++;
       any_size = true;
     }
@@ -792,13 +837,14 @@ bool MatchSignature::parse_conv_id (const char *&str, MatchTypeRef &type) {
   // decl_spec_seq_opt declarator_opt
   const char *curr = str;
   FctSpec fct_spec;
-  parse_decl_spec_seq (curr, type, fct_spec);
+  Spec spec;
+  parse_decl_spec_seq( curr, type, fct_spec, spec );
   if (error () || curr == str)
     return false;
   
   // function specifiers in conversion function IDs are not allowed
-  if (fct_spec) {
-    _state = ERR_INVALID_FCT_SPEC;
+  if( fct_spec || spec ) {
+    _state = ERR_INVALID_SPEC;
     return false;
   }
   
@@ -889,8 +935,20 @@ bool MatchSignature::parse_operator_id (const char *&str, MatchName::Operator &o
       op = MatchName::OP_DIV;
   }
   else if (first == '%') { // %, %%, %%= ; that's ugly! %% has to be used for %
-    if (*str == '%') {
-      str++;
+    if (_wildcards_allowed) {
+      if (*str == '%') {
+        str++;
+        if (*str == '=') {
+          str++;
+          op = MatchName::OP_MOD_EQ;
+        }
+        else
+          op = MatchName::OP_MODULO;
+      }
+      else
+        op = MatchName::OP_ANY;
+    }
+    else {
       if (*str == '=') {
         str++;
         op = MatchName::OP_MOD_EQ;
@@ -898,8 +956,6 @@ bool MatchSignature::parse_operator_id (const char *&str, MatchName::Operator &o
       else  
         op = MatchName::OP_MODULO;
     }
-    else 
-      op = MatchName::OP_ANY;
   }
   else if (first == '^') { // ^, ^=
     if (*str == '=') {
@@ -996,8 +1052,20 @@ bool MatchSignature::parse_operator_id (const char *&str, MatchName::Operator &o
       op = MatchName::OP_CALL;
     }
   }
-  else if (first == ',') { // ,
-    op = MatchName::OP_COMMA;
+  else if (first == '?') { // ?:
+    if (*str == ':') {
+      str++;
+      op = MatchName::OP_COND;
+    }
+  }
+  else if (first == '.') { // ., .*
+    if (*str == '*') {
+      str++;
+      op = MatchName::OP_DOT_STAR;
+    }
+    else {
+      op = MatchName::OP_DOT;
+    }
   }
 
   if (op != MatchName::OP_UNDEFINED)
@@ -1096,5 +1164,5 @@ inline bool MatchSignature::next_word (const char *word, const char *&str) const
 
 inline bool MatchSignature::is_id (char c) const {
   return ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') ||
-          c == '_' || c == '%' || (c >= '0' && c <= '9'));
+          c == '_' || (_wildcards_allowed && c == '%') || (c >= '0' && c <= '9'));
 }
