@@ -36,7 +36,8 @@ string signature(const ACM_Name &name_param) {
   }
   result += name.get_name();
 
-  if (name.type_val() & JPT_Function) {
+  // the result so far seems to be a sane default so no explicit error handling neccesary, if below checks fall through
+  if( name.type_val() == JPT_Function ) {
     ACM_Function &func = (ACM_Function&)name;
     list<ACM_Type*> types;
     get_arg_types (func, types);
@@ -53,26 +54,32 @@ string signature(const ACM_Name &name_param) {
       result += " const";
     if (func.get_cv_qualifiers() & CVQ_VOLATILE)
       result += " volatile";
-    if (((ACM_Function &)name).has_result_type()) {
-      ACM_Type &result_type = *((ACM_Function &)name).get_result_type();
-      if (name.get_name().find("operator ") != 0)
+    if (func.has_result_type()) {
+      ACM_Type &result_type = *func.get_result_type();
+      if (func.get_name().find("operator ") != 0)
         result = format_type (result_type, result);
       else {
         string result_str = format_type(result_type);
-        if (name.get_name().find(string("operator ") + result_str) != 0)
+        if (func.get_name().find(string("operator ") + result_str) != 0)
           result = format_type (result_type, result);
       }
     }
   }
+  else if( name.type_val() == JPT_Variable ) {
+    ACM_Variable &var = (ACM_Variable &)name;
+
+    ACM_Type &type = *( var.get_type() );
+    result = format_type( type, result );
+  }
   return result;
 }
 
-string signature(const ACM_Code &code_param) {
-  ACM_Code &code = (ACM_Code&)code_param;
-  if (code.type_val() == JPT_Call)
-    return signature (*((ACM_Call&)code).get_target());
+string signature( const ACM_Code &code ) {
+  // signature of a code joinpoint is defined as signature of the matching entity
+  if( has_entity( code ) )
+    return signature( *get_entity( code ) );
   else
-    return signature (*(ACM_Name*)code.get_parent());
+    return format_type( *get_entity_type( code ), "<unknown>" );
 }
 
 string format_type(ACM_Type &type) {
@@ -105,28 +112,133 @@ string format_type(ACM_Type &type, const string &name) {
   return sig; 
 }
 
+// functions that return the entity for certain join-points
+bool has_entity( const ACM_Code &jpl ) {
+  return ( has_entity_func( jpl ) || has_entity_var( jpl ) );
+}
+
+ACM_Name *get_entity( const ACM_Code &jpl ) {
+  if( has_entity_func( jpl ) ) // Code joinpoints that have a function as entity
+    return get_entity_func( jpl );
+  else if ( has_entity_var( jpl ) ) // Code joinpoints that have a variable as entity
+    return get_entity_var( jpl );
+  else { // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+    return 0; // we need to return something if compiled without asserts
+  }
+}
+
+bool has_entity_func( const ACM_Code &jpl ) {
+  // Code joinpoints that have a function as entity:
+  // JPT_Call
+  // JPT_Builtin
+  // JPT_Execution
+  // JPT_Construction
+  // JPT_Destruction
+  return ( jpl.type_val() & ( JPT_Call | JPT_Builtin | JPT_Execution | JPT_Construction | JPT_Destruction ) );
+}
+
+ACM_Function *get_entity_func( const ACM_Code &jpl ) {
+  switch( jpl.type_val() ) {
+  case JPT_Call:
+    return const_cast<ACM_Call &>( static_cast<const ACM_Call &>( jpl ) ).get_target(); // for some reason the getter is not defined const, but should be
+  case JPT_Builtin:
+    return const_cast<ACM_Builtin &>( static_cast<const ACM_Builtin &>( jpl ) ).get_target();
+  case JPT_Execution: // inner joinpoints have there entity as parent
+  case JPT_Construction:
+  case JPT_Destruction:
+    return static_cast<ACM_Function *>( jpl.get_parent() );
+  default: // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+    return 0; // we need to return something if compiled without asserts
+  }
+}
+
+bool has_entity_var( const ACM_Code &jpl ) {
+  // Code joinpoints that have a variable as entity:
+  // JPT_Get
+  // JPT_Set
+  // JPT_Ref
+  return ( jpl.type_val() & ( JPT_Get | JPT_Set | JPT_Ref ) );
+};
+
+ACM_Variable *get_entity_var( const ACM_Code &jpl ) {
+  switch( jpl.type_val() ) {
+  case JPT_Get:
+    return const_cast<ACM_Get &>( static_cast<const ACM_Get &>( jpl ) ).get_variable(); // for some reason the getter is not defined const, but should be
+  case JPT_Set:
+    return const_cast<ACM_Set &>( static_cast<const ACM_Set &>( jpl ) ).get_variable(); // for some reason the getter is not defined const, but should be
+  case JPT_Ref:
+    return const_cast<ACM_Ref &>( static_cast<const ACM_Ref &>( jpl ) ).get_variable(); // for some reason the getter is not defined const, but should be
+  default: // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+    return 0; // we need to return something if compiled without asserts
+  }
+}
+
+ACM_Type *get_entity_type( const ACM_Code &jpl ) {
+  switch( jpl.type_val() ) {
+  case JPT_Get:
+    // FIXME: if the get join point accesses an array, the entity type must be the element type (and not the array type)
+    return const_cast<ACM_Get &>( static_cast<const ACM_Get &>( jpl ) ).get_variable()->get_type(); // for some reason the getter is not defined const, but should be
+  case JPT_Set:
+    // FIXME: if the set join point accesses an array, the entity type must be the element type (and not the array type)
+    return const_cast<ACM_Set &>( static_cast<const ACM_Set &>( jpl ) ).get_variable()->get_type(); // for some reason the getter is not defined const, but should be
+  case JPT_Ref:
+    return const_cast<ACM_Ref &>( static_cast<const ACM_Ref &>( jpl ) ).get_variable()->get_type(); // for some reason the getter is not defined const, but should be
+  case JPT_GetRef:
+    return const_cast<ACM_GetRef &>( static_cast<const ACM_GetRef &>( jpl ) ).get_type();
+  case JPT_SetRef:
+    return const_cast<ACM_SetRef &>( static_cast<const ACM_SetRef &>( jpl ) ).get_type();
+  default: // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+    return 0; // we need to return something if compiled without asserts
+  }
+}
+
 // functions that return the number of argument types
 
 int get_arg_count (ACM_Function &func) {
   return func.get_arg_types().size();
 }
 
-int get_arg_count (ACM_Call &code) {
-  ACM_Function *func = code.get_target();
-  if (code.has_default_args())
-    return get_arg_count (*func) - code.get_default_args();
+int get_arg_count (ACM_Call &call) {
+  ACM_Function *func = call.get_target();
+  if (call.has_default_args())
+    return get_arg_count (*func) - call.get_default_args();
   else
-    return get_arg_count (*func) + code.get_variadic_arg_types().size();
+    return get_arg_count (*func) + call.get_variadic_arg_types().size();
 }
 
 int get_arg_count (ACM_Code &code) {
-  if (code.type_val() == JPT_Call)
-    return get_arg_count ((ACM_Call&)code);
-  else
-    return get_arg_count (*(ACM_Function*)code.get_parent());
+  switch( code.type_val() ) {
+  // first cases which have an associated function
+  case JPT_Call: // handle seperately as adjustments are needed
+    return get_arg_count( static_cast<ACM_Call &>( code ) );
+  case JPT_Builtin:
+  case JPT_Execution:
+  case JPT_Construction:
+  case JPT_Destruction:
+    return get_arg_count( *get_entity_func( code ) );
+  // cases that behave like a call but are none
+  // currently these values need to be hard coded
+  // JPT_Get( T x ) -> T get()
+  case JPT_Get:
+  case JPT_GetRef:
+    return 0;
+  // JPT_Set( T x ) -> void set( T val )
+  case JPT_Set:
+  case JPT_SetRef:
+    return 1;
+  // JPT_Ref( T x ) -> T(&|*) ref()
+  case JPT_Ref:
+    return 0;
+  default: // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+    return 0; // we need to return something if compiled without asserts
+  }
 }
 
-
 // functions that return the argument/result types of certain join-points
 
 void add_arg_types (ACM_Function &func, list<ACM_Type*> &type_list) {
@@ -164,12 +276,35 @@ void get_arg_types (ACM_Call &jpl, list<ACM_Type*> &type_list) {
 }
 
 
-void add_arg_types (ACM_Code &jpl, list<ACM_Type*> &type_list) {
-  // TODO: this type check is dangerous!
-  if (jpl.type_val() == JPT_Call)
-    add_arg_types ((ACM_Call&)jpl, type_list);
-  else
-    add_arg_types (*(ACM_Function*)jpl.get_parent(), type_list);
+void add_arg_types( ACM_Code &jpl, list<ACM_Type *> &type_list ) {
+  switch( jpl.type_val() ) {
+  // first cases which have an associated function
+  case JPT_Call: // handle seperately as adjustments are needed
+    add_arg_types( static_cast<ACM_Call &>( jpl ), type_list );
+    return;
+  case JPT_Builtin:
+  case JPT_Execution:
+  case JPT_Construction:
+  case JPT_Destruction:
+    add_arg_types( *get_entity_func( jpl ), type_list );
+    return;
+  // cases that behave like a call but are none
+  // currently these values need to be hard coded
+  // JPT_Get( T x ) -> T get()
+  case JPT_Get:
+  case JPT_GetRef:
+    return; // no args
+  // JPT_Set( T x ) -> void set( T val )
+  case JPT_Set:
+  case JPT_SetRef:
+    type_list.push_back( get_entity_type( jpl ) ); // only arg has type of variable
+    return;
+  // JPT_Ref( T x ) -> T(&|*) ref()
+  case JPT_Ref:
+    return; // no args
+  default: // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+  }
 }
 
 void get_arg_types (ACM_Code &jpl, list<ACM_Type*> &type_list) {
@@ -177,18 +312,60 @@ void get_arg_types (ACM_Code &jpl, list<ACM_Type*> &type_list) {
   add_arg_types (jpl, type_list);
 }
 
- bool has_result_type (ACM_Code &jpl) {
-  if (jpl.type_val() == JPT_Call)
-    return ((ACM_Call&)jpl).get_target()->has_result_type();
-  else
-    return ((ACM_Function*)jpl.get_parent())->has_result_type();
+bool has_result_type( ACM_Code &jpl ) {
+  switch( jpl.type_val() ) {
+  // first cases which have an associated function
+  case JPT_Call:
+  case JPT_Builtin:
+  case JPT_Execution:
+  case JPT_Construction:
+  case JPT_Destruction:
+    return get_entity_func( jpl )->has_result_type();
+  // cases which operate on data
+  // currently these values need to be hard coded
+  // JPT_Get( T x ) -> T get()
+  case JPT_Get:
+  case JPT_GetRef:
+    return true;
+  // JPT_Set( T x ) -> void set( T val )
+  case JPT_Set:
+  case JPT_SetRef:
+    return false;
+  // JPT_Ref( T x ) -> T(&|*) ref()
+  case JPT_Ref:
+    return true;
+  default: // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+    return false; // something needs to be returned for builds without asserts
+  }
 }
 
-ACM_Type *get_result_type (ACM_Code &jpl) {
-  if (jpl.type_val() == JPT_Call)
-    return ((ACM_Call&)jpl).get_target()->get_result_type();
-  else
-    return ((ACM_Function*)jpl.get_parent())->get_result_type();
+ACM_Type *get_result_type( ACM_Code &jpl ) {
+  switch( jpl.type_val() ) {
+  // first cases which have an associated function
+  case JPT_Call:
+  case JPT_Builtin:
+  case JPT_Execution:
+  case JPT_Construction:
+  case JPT_Destruction:
+    return get_entity_func( jpl )->get_result_type();
+  // cases which operate on data
+  // currently these values need to be hard coded
+  // JPT_Get( T x ) -> T get()
+  case JPT_Get:
+  case JPT_GetRef:
+    return get_entity_type( jpl );
+  // JPT_Set( T x ) -> void set( T val )
+  case JPT_Set:
+  case JPT_SetRef:
+    return 0;
+  // JPT_Ref( T x ) -> T(&|*) ref()
+  case JPT_Ref:
+    return const_cast<ACM_Ref &>( static_cast<const ACM_Ref &>( jpl ) ).get_type(); // for some reason the getter is not defined const, but should be
+  default: // illegal or new JPType -> make noise
+    assert( false && "illegal and thus unhandled JPType" );
+    return 0; // something needs to be returned for builds without asserts
+  }
 }
 
 // check whether two functions have the same name and argument types
@@ -214,10 +391,27 @@ bool is_pseudo(ACM_Any &jpl) {
   return false;
 }
 
+// check if the given joinpoint is an implicit one
+bool is_implicit( ACM_Code &jpl ) {
+  ACM_Node *parent = static_cast<ACM_Node *>( jpl.get_parent() );
+  return parent && ( parent->type_val() & JPT_Code );
+}
+
+// get the parent of the explit joinpoint in code
+ACM_Name *get_explicit_parent( ACM_Code &jpl ) {
+  ACM_Code *code = &jpl;
+  while( code && is_implicit( *code ) )
+    code = static_cast<ACM_Code *>( code->get_parent() );
+
+  assert( ! code || ( static_cast<ACM_Node *>( code->get_parent() )->type_val() & JPT_Name ) );
+  return code ? static_cast<ACM_Name *>( code->get_parent() ) : 0;
+}
+
 // get the lexical scope of a code join-point as needed by 'within'
 ACM_Name *lexical_scope (ACM_Code &jpl) {
-  ACM_Name *parent = (ACM_Name*)jpl.get_parent();
-  if (jpl.type_val() == JPT_Call) {
+  ACM_Name *parent = get_explicit_parent( jpl );
+  assert( parent );
+  if (jpl.type_val() & JPT_Access) {
     if (parent->type_val() == JPT_Variable)
       parent = (ACM_Name*)parent->get_parent();
   }
@@ -226,11 +420,21 @@ ACM_Name *lexical_scope (ACM_Code &jpl) {
   return parent;
 }
 
+// check whether a given program entity needs an object to be meaningful
+bool needs_this( ACM_Name &entity ) {
+  if( entity.type_val() == JPT_Function )
+    return needs_this( static_cast<ACM_Function &>( entity ) );
+  else if( entity.type_val() == JPT_Variable )
+    return needs_this( static_cast<ACM_Variable &>( entity ) );
+  else {
+    assert( false && "Unknown and/or not supported entity type !" );
+    return false; // something needs to be returned for builds without asserts
+  }
+}
+
 // check whether a function needs an object to be invoked ("this pointer")
 bool needs_this (ACM_Function &func) {
-  assert (func.get_kind() != FT_UNKNOWN);
   switch (func.get_kind()) {
-  case FT_UNKNOWN:
   case FT_NON_MEMBER:
   case FT_STATIC_NON_MEMBER:
   case FT_STATIC_MEMBER:
@@ -243,13 +447,32 @@ bool needs_this (ACM_Function &func) {
   case FT_MEMBER:
   case FT_VIRTUAL_MEMBER:
   case FT_PURE_VIRTUAL_MEMBER:
-    break;
-  }
-  if (func.get_name().substr(0,9) != "operator ")
-    return true;
+    {
+    if( ! is_operator( &func ) )
+      return true;
 
-  string name = func.get_name().substr(9);
-  return name != "new" && name != "new[]" && name != "delete" && name != "delete[]";
+    string name = func.get_name().substr(9);
+    return name != "new" && name != "new[]" && name != "delete" && name != "delete[]";
+    }
+  default: // illegal or new FType -> make noise
+    assert( false && "illegal and thus unhandled FType" );
+    return false; // something needs to be returned for builds without asserts
+  }
+}
+
+// check whether a variable is part of an object, thus needs a this pointer for an access
+bool needs_this( ACM_Variable &var ) {
+  switch( var.get_kind() ) {
+  case VT_NON_MEMBER:
+  case VT_STATIC_NON_MEMBER:
+  case VT_STATIC_MEMBER:
+    return false;
+  case VT_MEMBER:
+    return true;
+  default: // illegal or new VType -> make noise
+    assert( false && "illegal and thus unhandled VType" );
+    return false; // something needs to be returned for builds without asserts
+  }
 }
 
 // check whether a name joinpoint is defined in the project
@@ -267,19 +490,17 @@ bool proceed_needs_result (ACM_Code &jpl) {
 }
 
 bool proceed_needs_that (ACM_Code &jpl) {
-  switch (jpl.type_val()) {
-  case JPT_Construction:
-  case JPT_Destruction:
-    return true;
-  case JPT_Execution:
-    return needs_this(*(ACM_Function*)((ACM_Execution&)jpl).get_parent());
-  default:
-    return false;
-  }
+  if( jpl.type_val() & JPT_Access ) // outer joinoints ...
+    return false; // ... never need it
+  else // inner joinpoints ...
+    return needs_this( *get_entity( jpl ) ); // determine by entity
 }
 
 bool proceed_needs_target (ACM_Code &jpl) {
-  return jpl.type_val() == JPT_Call && needs_this(*((ACM_Call&)jpl).get_target());
+  if( jpl.type_val() & JPT_Access ) // outer joinoints ...
+    return needs_this( *get_entity( jpl ) ); // determine by entity
+  else // inner joinpoints ...
+    return false; // ... never need it
 }
 
 // manage child name map of ACM_Name nodes
@@ -435,3 +656,9 @@ bool is_abstract (ACM_Class &cls) {
 int depth (ACM_CodePlan &plan) {
   return plan.has_next_level() ? depth (*plan.get_next_level()) + 1 : 0;
 }
+
+// This function takes an AspectC++-Model-Function as ACM_Function and
+// returns whether this function is an operator.
+bool is_operator(ACM_Function* acm_function) {
+  return acm_function->get_name().substr(0,9) == "operator ";
+}
