@@ -19,12 +19,17 @@
 #ifndef __PointCutExpr_h__
 #define __PointCutExpr_h__
 
-#include "Puma/Array.h"
-#include "Puma/Location.h"
-using namespace Puma;
+#ifdef FRONTEND_PUMA
+namespace Puma {
+  class CTypeInfo;
+}
+#else // FRONTEND_CLANG
+#include "clang/AST/Type.h"
+#endif
 
 #include "Condition.h"
 #include "PointCut.h"
+#include "ACErrorStream.h"
 
 #include "ACModel/MatchExpr.h"
 #include "ACModel/Elements.h"
@@ -32,15 +37,11 @@ using namespace Puma;
 
 #include <list>
 #include <string>
+#include "Puma/Array.h" // TODO: replace by STL container
 
 class Binding;
 class PointCutContext;
 
-namespace Puma {
-  class CTypeInfo;
-  class ErrorStream;
-} // namespace Puma
-
 // Pointcut expression types are evaluated in a context-independant manner.
 // For every pointcut expression a type can always be determined.
 enum PCE_Type { 
@@ -48,10 +49,7 @@ enum PCE_Type {
   PCE_CODE // CODE represents a code pointcut
 };
 
-
-class PointCutExpr {
-
-  int _ref_count;
+class PointCutExpr : public PointcutExpressionNode {
 
 protected:
 
@@ -60,32 +58,25 @@ protected:
   JoinPointType _possible_types;
 
   // do a semantic analysis of the child nodes
-  void sem_args (ErrorStream &err, PointCutContext &context);
+  void sem_args (ACErrorStream &err, PointCutContext &context, int warn_compat);
 
   // check if the argument pointcut types are as expected
-  void check_arg_types (ErrorStream &err, const char *func, PCE_Type expected,
+  void check_arg_types (ACErrorStream &err, const char *func, PCE_Type expected,
       PointCutContext &context);
 
   // check if all argument types are equal
-  void check_arg_types_equal (ErrorStream &err, const char *func,
+  void check_arg_types_equal (ACErrorStream &err, const char *func,
       PointCutContext &context);
 
   // get the real argument binding of a context variable
-  const ACM_Arg *get_binding (const char *name, ErrorStream &err,
+  const ACM_Arg *get_binding (const char *name, ACErrorStream &err,
       PointCutContext &context);
 public:
 
-  PointCutExpr () : _ref_count (1), _pre_evaluate (PRE_EVAL_OFF) {}
+  PointCutExpr () : _pre_evaluate (PRE_EVAL_OFF), _jpl (0),
+    _possible_types((JoinPointType)0) {}
   virtual ~PointCutExpr () {}
 
-  // get/modify the reference count
-  int get_ref () const { return _ref_count; }
-  void ref(int delta = 1) { _ref_count += delta; }
-
-  // set and get the corresponding syntax tree and expression child nodes
-  virtual int args () const = 0;
-  virtual PointCutExpr *arg (int) const = 0;
-
   // Return the type of pointcut described by this expression. This function
   // does not check any argument types etc. -> semantics!
   virtual PCE_Type type () const = 0;
@@ -94,7 +85,8 @@ public:
   int possible_types () const { return (int)_possible_types; }
 
   // run a semantic analysis (e.g. check types)
-  virtual void semantics (ErrorStream &err, PointCutContext &context) = 0;
+  virtual void semantics (ACErrorStream &err, PointCutContext &context,
+      int warn_compat) = 0;
 
   // evaluate an expression, i.e. check if the expression evaluates as 'true'
   // for a particular join point
@@ -116,22 +108,9 @@ public:
   // mangle a string that represents a type check (for that/target arguments)                         
   virtual void mangle_type_check (ostream &) = 0;
 
-  // duplicate the whole pointcut expression tree -- at least logically
+  // helper functions to get the right result type
   virtual PointCutExpr* duplicate () = 0;
-
-  // destroy a pointcut expression tree
-  // TODO: not a really intuitive solution, find a better one ...
-  static void destroy (PointCutExpr * expr) {
-    if (!expr) return;
-    expr->ref(-1);
-    if (expr->get_ref () == 0) {
-      for (int i = 0; i < expr->args (); i++)
-        destroy (expr->arg (i));
-      delete expr;
-    }
-  }
-
-
+  virtual PointCutExpr *arg (int i) const = 0;
 };
 
 class PCE_SingleArg : public PointCutExpr {
@@ -147,10 +126,14 @@ public:
     PointCutContext &context, Binding &binding, Condition &cond);
   bool check_derived_func (ACM_Class &cls, ACM_Function &func, 
     PointCutContext &context, Binding &binding, Condition &cond);
+  bool check_derived_var (ACM_Class &cls, ACM_Variable &var,
+    PointCutContext &context, Binding &binding, Condition &cond);
   bool check_base_class (ACM_Class &cls, PointCutContext &context,
     Binding &binding, Condition &cond);
   bool check_base_func (ACM_Class &cls, ACM_Function &func, 
     PointCutContext &context, Binding &binding, Condition &cond);
+  bool check_base_var (ACM_Class &cls, ACM_Variable &var,
+    PointCutContext &context, Binding &binding, Condition &cond);
   bool check_scopes (ACM_Name *scope,
     PointCutContext &context, Binding &binding, Condition &cond);
   
@@ -178,7 +161,7 @@ public:
 };
 
 class PCE_VarArgs : public PointCutExpr {
-  Array<PointCutExpr*> _args;
+  Puma::Array<PointCutExpr*> _args; // TODO: replace by STL container
 public:
   virtual ~PCE_VarArgs () {}
   void add_arg (PointCutExpr *arg) { _args.append (arg); }
@@ -195,7 +178,7 @@ class PCE_Classes : public PCE_SingleArg {
 public:
   PCE_Classes (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -209,7 +192,7 @@ class PCE_Base : public PCE_SingleArg {
 public:
   PCE_Base (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -223,7 +206,7 @@ class PCE_Derived : public PCE_SingleArg {
 public:
   PCE_Derived (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -237,7 +220,7 @@ class PCE_Within : public PCE_SingleArg {
 public:
   PCE_Within (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -247,11 +230,24 @@ public:
   virtual PCE_Within* duplicate () { return new PCE_Within(arg(0)->duplicate ()); }
 };
 
+class PCE_Member : public PCE_SingleArg {
+public:
+  PCE_Member( PointCutExpr *arg ) : PCE_SingleArg( arg ) {}
+  virtual PCE_Type type() const;
+  virtual void semantics( ACErrorStream &err, PointCutContext &context, int warn_compat);
+  virtual bool evaluate( ACM_Any &jpl, PointCutContext &context, Binding &binding, Condition &cond );
+  virtual void mangle_type_check( ostream &out ) {
+    out << "member_";
+    PCE_SingleArg::mangle_type_check( out );
+  }
+  virtual PCE_Member* duplicate () { return new PCE_Member( arg(0)->duplicate () ); }
+};
+
 class PCE_Execution : public PCE_SingleArg {
 public:
   PCE_Execution (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -265,7 +261,7 @@ class PCE_Call : public PCE_SingleArg {
 public:
   PCE_Call (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -275,11 +271,81 @@ public:
   virtual PCE_Call* duplicate () { return new PCE_Call(arg(0)->duplicate ()); }
 };
 
+class PCE_Builtin : public PCE_SingleArg {
+public:
+  PCE_Builtin (PointCutExpr *arg) : PCE_SingleArg (arg) {}
+  virtual PCE_Type type () const;
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
+  virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
+                         Binding &binding, Condition &cond);
+  virtual void mangle_type_check (ostream &out) {
+    out << "builtin";
+    PCE_SingleArg::mangle_type_check (out);
+  }
+  virtual PCE_Builtin* duplicate () { return new PCE_Builtin(arg(0)->duplicate ()); }
+};
+
+class PCE_Get : public PCE_SingleArg {
+public:
+  PCE_Get( PointCutExpr *arg ) : PCE_SingleArg( arg ) {}
+  virtual PCE_Type type () const;
+  virtual void semantics( ACErrorStream &err, PointCutContext &context, int warn_compat);
+  virtual bool evaluate( ACM_Any &jpl, PointCutContext &context,
+                         Binding &binding, Condition &cond );
+  virtual void mangle_type_check( ostream &out ) {
+    out << "get_";
+    PCE_SingleArg::mangle_type_check( out );
+  }
+  virtual PCE_Get* duplicate() { return new PCE_Get( arg(0)->duplicate() ); }
+};
+
+class PCE_Set : public PCE_SingleArg {
+public:
+  PCE_Set( PointCutExpr *arg ) : PCE_SingleArg( arg ) {}
+  virtual PCE_Type type () const;
+  virtual void semantics( ACErrorStream &err, PointCutContext &context, int warn_compat);
+  virtual bool evaluate( ACM_Any &jpl, PointCutContext &context,
+                         Binding &binding, Condition &cond );
+  virtual void mangle_type_check( ostream &out ) {
+    out << "set_";
+    PCE_SingleArg::mangle_type_check( out );
+  }
+  virtual PCE_Set* duplicate() { return new PCE_Set( arg(0)->duplicate() ); }
+};
+
+class PCE_Ref : public PCE_SingleArg {
+public:
+  PCE_Ref( PointCutExpr *arg ) : PCE_SingleArg( arg ) {}
+  virtual PCE_Type type () const;
+  virtual void semantics( ACErrorStream &err, PointCutContext &context, int warn_compat);
+  virtual bool evaluate( ACM_Any &jpl, PointCutContext &context,
+                         Binding &binding, Condition &cond );
+  virtual void mangle_type_check( ostream &out ) {
+    out << "ref_";
+    PCE_SingleArg::mangle_type_check( out );
+  }
+  virtual PCE_Ref* duplicate() { return new PCE_Ref( arg(0)->duplicate() ); }
+};
+
+class PCE_Alias : public PCE_SingleArg {
+public:
+  PCE_Alias( PointCutExpr *arg ) : PCE_SingleArg( arg ) {}
+  virtual PCE_Type type () const;
+  virtual void semantics( ACErrorStream &err, PointCutContext &context, int warn_compat);
+  virtual bool evaluate( ACM_Any &jpl, PointCutContext &context,
+                         Binding &binding, Condition &cond );
+  virtual void mangle_type_check( ostream &out ) {
+    out << "alias_";
+    PCE_SingleArg::mangle_type_check( out );
+  }
+  virtual PCE_Alias* duplicate() { return new PCE_Alias( arg(0)->duplicate() ); }
+};
+
 class PCE_Construction : public PCE_SingleArg {
 public:
   PCE_Construction (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -293,7 +359,7 @@ class PCE_Destruction : public PCE_SingleArg {
 public:
   PCE_Destruction (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -307,7 +373,7 @@ class PCE_That : public PCE_SingleArg {
 public:
   PCE_That (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -321,7 +387,7 @@ class PCE_Target : public PCE_SingleArg {
 public:
   PCE_Target (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -339,7 +405,7 @@ public:
   virtual ~PCE_CFlow () {}
   virtual PCE_Type type () const;
   PointCut &arg_pointcut () { return _arg_pointcut; }
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -350,10 +416,18 @@ public:
 };
 
 class PCE_Args : public PCE_VarArgs {
+  bool _both_sc_args_bound_warning_created;
+  bool _sc_arg_bound_warning_created;
 public:
+  PCE_Args () : _both_sc_args_bound_warning_created(false),
+                _sc_arg_bound_warning_created(false) {}
+  PCE_Args (bool both_sc_args_bound_warning_created,
+            bool sc_arg_bound_warning_created) :
+        _both_sc_args_bound_warning_created(both_sc_args_bound_warning_created),
+        _sc_arg_bound_warning_created(sc_arg_bound_warning_created){}
   virtual ~PCE_Args () {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -361,7 +435,8 @@ public:
     PCE_VarArgs::mangle_type_check (out);
   }
   virtual PCE_Args* duplicate () {
-    PCE_Args *result = new PCE_Args;
+    PCE_Args *result = new PCE_Args(_both_sc_args_bound_warning_created,
+        _sc_arg_bound_warning_created);
     for (int i = 0; i < args(); i++)
       result->add_arg(arg(i)->duplicate ());
     return result;
@@ -372,7 +447,7 @@ class PCE_Result : public PCE_SingleArg {
 public:
   PCE_Result (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -387,7 +462,7 @@ public:
   PCE_Or (PointCutExpr *arg0, PointCutExpr *arg1) : 
     PCE_DoubleArg (arg0, arg1) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -402,7 +477,7 @@ public:
   PCE_And (PointCutExpr *arg0, PointCutExpr *arg1) : 
     PCE_DoubleArg (arg0, arg1) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -416,7 +491,7 @@ class PCE_Not : public PCE_SingleArg {
 public:
   PCE_Not (PointCutExpr *arg) : PCE_SingleArg (arg) {}
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
@@ -426,23 +501,27 @@ public:
   virtual PCE_Not* duplicate () { return new PCE_Not(arg(0)->duplicate ()); }
 };
 
-class PCE_Named : public PCE_SingleArg {
+class PCE_Named : public PointCutExpr {
   ACM_Pointcut *_pct_func;
+  PointcutExpression _expr;
   std::list<std::string> _params;
 public:
-  PCE_Named (ACM_Pointcut *pct_func, PointCutExpr *expr) :
-    PCE_SingleArg (expr), _pct_func (pct_func) {}
+  PCE_Named (ACM_Pointcut *pct_func) : _pct_func (pct_func) {}
   virtual PCE_Type type () const;
   std::list<std::string> &params () { return _params; }
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  PointcutExpression &expr () { return _expr; }
+  virtual int args () const { return 0; }
+  virtual PointCutExpr *arg (int) const { return 0; }
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out) {
     out << "named_";
-    arg(0)->mangle_type_check (out);
+    ((PointCutExpr*)_expr.get())->mangle_type_check (out);
   }
   virtual PCE_Named* duplicate () {
-    PCE_Named *result = new PCE_Named(_pct_func, arg (0) ? arg(0)->duplicate() : 0);
+    PCE_Named *result = new PCE_Named(_pct_func);
+    result->_expr = _expr;
     result->_params = _params;
     return result;
   }
@@ -451,7 +530,11 @@ public:
 class PCE_ContextVar : public PointCutExpr {
   string _name;
   const ACM_Arg *_bound_arg;
-  const CTypeInfo *_check_type;
+#ifdef FRONTEND_PUMA  // TODO: get rid of frontend types here
+  const Puma::CTypeInfo *_check_type;
+#else // FRONTEND_CLANG
+  clang::QualType _check_type;
+#endif
   enum { CV_THAT, CV_TARGET, CV_ARG, CV_RESULT } _bound_to;
   int _arg;
 public:
@@ -459,7 +542,7 @@ public:
   virtual int args () const { return 0; }
   virtual PointCutExpr *arg (int) const { return 0; }
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
   virtual bool evaluate (ACM_Any &jpl, PointCutContext &context,
                          Binding &binding, Condition &cond);
   virtual void mangle_type_check (ostream &out);
@@ -475,7 +558,8 @@ public:
   virtual int args () const { return 0; }
   virtual PointCutExpr *arg (int) const { return 0; }
   virtual PCE_Type type () const;
-  virtual void semantics (ErrorStream &err, PointCutContext &context);
+  virtual void semantics (ACErrorStream &err, PointCutContext &context, int warn_compat);
+  bool parse ();
 
   // if the expression supports pre-evaluation, do it in this function
   virtual void pre_evaluate (PointCutContext &context);
