@@ -1,4 +1,5 @@
 #include <iostream>
+#include <inttypes.h>
 using namespace std;
 
 // =====================================================================
@@ -168,7 +169,12 @@ struct Base2 {
 };
 
 struct Derived : public Base1, public virtual Base2 {
-  const char *k;
+  const char *k; // non-const
+  //int* const c; // const
+  const int c2; // const
+  void cee() const {} // const
+  const int zee() { return 0; } // non-const
+  Derived() : /*c((int*)this),*/ c2(0xCAFEBABE) {}
 };
 
 // arbitrary members ...
@@ -202,6 +208,18 @@ aspect TypeExtensions {
   }
 };
 
+// template members as introduction
+namespace Test {
+template<typename T>
+class A {};
+};
+
+class B {};
+
+aspect Introducer {
+  advice "B" : slice class { Test::A<JoinPoint> a; };
+};
+
 // -------
 // Bug 469: enums are counted for MEMBERS, but not treated in the list itself
 
@@ -236,6 +254,102 @@ extern "C" {
   };
 }
 
+// -------
+// Bug 525: introspection code for members that are instances of anonymous types
+//          -> fine if it compiles
+
+class Bug525_1 {
+  struct
+  {
+    bool     hit  : 1;
+    unsigned res  : 6;
+  } _flags;
+  typedef struct { int i; } IStruct;
+  IStruct _is;
+};
+
+class Bug525_2 {
+union
+    {
+      struct
+	{
+	  unsigned long _space:32;
+	  unsigned long address:20;
+        } __attribute__((packed)) data;
+      int *_submap;
+    } __attribute__((packed));
+};
+
+// don't de-anonymize typedef'd classes
+class Bug525_3 {
+public:
+  Bug525_3 () {}
+};
+
+typedef class { Bug525_3 _attr; } Bug525_4;
+Bug525_4 global_obj;
+
+struct {
+  Bug525_4 _obj;
+  int _bla;
+} typedef Bug525_5;
+
+void f(const Bug525_5 &p) {}
+aspect Bug525_Check {
+  advice execution("void f(const %&)") : before () {}
+};
+
+// -------
+// Bug 546:  Type hiding by a non-type declaration
+//           -> requires elaborated type specifier
+struct settings {
+  int hashpower_init;
+};
+
+class Memcached {
+  static struct settings settings;
+  const volatile struct settings hmpf;
+  void (Memcached::*wr)(char const *, int, unsigned & );
+};
+
+settings Memcached::settings;
+
+// -------
+// Bug 524:  handle ambiguous type names
+//
+
+// case 1
+namespace Addr {
+
+template< int SHIFT >
+struct Addr {};
+
+} // namespace Addr
+
+class Virt_addr
+: public Addr::Addr<42> {};
+
+// case 2
+class Page
+{
+  typedef int Type;
+  Type t;
+};
+
+// case 3 (sanity check)
+namespace Inner {
+  struct T { uint32_t a; };
+}
+
+class C2 {
+  public:
+  class Inner {
+    public: struct T { uint8_t a; };
+  };
+  Inner::T my_t;
+  ::Inner::T my_t2;
+};
+
 // ======
 
 int main () {
@@ -259,5 +373,11 @@ int main () {
     print (d); cout << endl;
     cout << "-Bug-469------------------------------------------" << endl;
     print (bug469); cout << endl;
+    cout << "-Bug-525------------------------------------------" << endl;
+    cout << "#members of Bug525_1: " << AC::TypeInfo<Bug525_1>::MEMBERS << endl;
+    cout << "#members of Bug525_2: " << AC::TypeInfo<Bug525_2>::MEMBERS << endl;
+    f (Bug525_5 ());
+    cout << "Bug524: must be 1: " << sizeof(AC::TypeInfo<C2>::Member<0>::Type) << endl;
+    cout << "Bug524: must be 4: " << sizeof(AC::TypeInfo<C2>::Member<1>::Type) << endl;
     cout << "==================================================" << endl;
 }
